#include "delay.h"
#include "stat.h"

void
delay_buf_set(struct stat_rec *stat, double v, double env_v)
{
	int i;

	i = (stat->nt->ot->smp_cnt - stat->fst_smp_cnt) % DELAY_BUF_N;
	stat->delay_buf[0][i] = v;
	stat->delay_buf[1][i] = env_v;
}

double
tbl_lookup(double *tbl, int n, double d_idx)
{
	int i;

	i = (int)d_idx;
	d_idx -= i;
	i %= n;
	return MIX(tbl[i], tbl[(i+1)%n], d_idx);
}

double
delay_buf_lookup(struct stat_rec *stat, double sec, double *ret_env_v)
{
	double cycle;

	cycle = stat->nt->ot->smp_freq * sec - stat->fst_smp_cnt;
	if(ret_env_v) *ret_env_v = tbl_lookup(stat->delay_buf[1], DELAY_BUF_N, cycle);
	return tbl_lookup(stat->delay_buf[0], DELAY_BUF_N, cycle);
}

double
delay_out(struct stat_rec *stat, struct delay_rec *delay, double *ret_env_v)
{
	double v, sec;

	sec = stat->nt->ot->sec;
	if(sec - stat->nt->on_sec < delay->sec) return 0;
	v = delay_buf_lookup(stat, sec - delay->sec, ret_env_v) * delay->gain;
	if(ret_env_v) *ret_env_v *= delay->gain;
	return v;
}

double
chorus_out(struct stat_rec *stat)
{
	double sec,dsec, bak_sec;

	sec = stat->nt->ot->sec;
	dsec = sec - stat->nt->on_sec;
	bak_sec = 0.01 + 0.001 * sin(2 * M_PI * 0.8 * dsec);
	if(dsec < bak_sec) return 0;
	return delay_buf_lookup(stat, sec - bak_sec, NULL);
}

/* EOF */
