#include "filter.h"

void
filter_update(struct filter_rec *fl, struct filter_stat_rec *stat, double smp_t)
{
	double w0, alpha, cos_w0;

	if(fl->type == OFF) return;

	w0 = 2 * M_PI * fl->freq * smp_t;
	cos_w0 = cos(w0);
	alpha = sin(w0) / (2 * fl->Q);

	switch(fl->type){
	case LPF:
		stat->b1 = 1 - cos_w0;
		stat->b0 = stat->b2 = stat->b1 * 0.5;
		break;
	case HPF:
		stat->b1 = -(1 + cos_w0);
		stat->b0 = stat->b2 = -stat->b1 * 0.5;
		break;
	case BPF:
		stat->b0 = fl->Q * alpha;
		stat->b1 = 0;
		stat->b2 = -stat->b0;
		break;
	}
	stat->a0 = 1 + alpha;
	stat->a1 = -2 * cos_w0;
	stat->a2 = 1 - alpha;
	if(stat->a0 != 0) stat->div_a0 = 1 / stat->a0;
}

void
filter_init(struct filter_rec *fl, struct filter_stat_rec *stat, double smp_t)
{
	int i;

	for(i=0; i<4; i++) stat->in[i] = stat->out[i] = 0;
	stat->idx = 0;

	filter_update(fl, stat, smp_t);
}

double
filter_out(struct filter_stat_rec *stat, double in)
{
	double *out_p;
	int i_1, i_2;

	stat->in[stat->idx] = in;
	out_p = &stat->out[stat->idx];

	if(stat->a0 != 0){
		i_1 = (stat->idx + 4 - 1) & 3;
		i_2 = (i_1 + 4 - 1) & 3;
		*out_p = (stat->b0 * in
			+ stat->b1 * stat->in[i_1]
			+ stat->b2 * stat->in[i_2]
			- stat->a1 * stat->out[i_1]
			- stat->a2 * stat->out[i_2] ) * stat->div_a0;
	}else{
		*out_p = 0;
	}
	stat->idx = (stat->idx + 1) & 3;
	return *out_p;
}

/* EOF */
