#include "rd.h"
#include "util.h"
#include "out.h"
#include "note.h"
#include "stat.h"
#include "ch.h"

int
header(void)
{
	int v;

	if(!rd_str_chk(4, "MThd")) ERR("header id");
	if(rd_int(4) != 6) ERR("header size");
	if(rd_int(2) != 0) ERR("header format type");
	if(rd_int(2) != 1) ERR("header track num");
	if((v = rd_int(2)) & 0x8000) ERR("header division");
	return v; /* div4 */
}

void
data_out(struct out_rec *ot, double evt_sec)
{
	double vl, vr;
	int i;
	struct note_rec *nt;

	while((ot->sec = ot->smp_t * ot->smp_cnt) < evt_sec){
		vl = vr = 0;
		for(i=0; i<NOTE_BUF_N; i++){
			if(note_buf_is_free(i)) continue;
			nt = &note_buf[i];
			nt->ot = ot;
			if(note_out(nt, &vl, &vr)){
				stat_lst_free(nt->stat_lst);
				note_buf_free(i);
			}
		}
		out_do(ot, vl * (1.0 / 16));
		if(ot->ch_num > 1) out_do(ot, vr * (1.0 / 16));
		ot->smp_cnt++;
	}
}

void
bend_note_update(int ch, double sec)
{
	int i, j;
	struct note_rec *nt;
	struct vco_stat_rec *vco_stat;
	struct stat_rec *stat;

	for(i=0; i<NOTE_BUF_N; i++){
		if(note_buf_is_free(i)) continue;
		nt = &note_buf[i];
		if(nt->ch != ch) continue;
		for(stat=nt->stat_lst; stat; stat=stat->next){
			for(j=0; j<2; j++){
				vco_stat = &stat->vco[j];
				vco_stat_update(vco_stat, vco_stat->freq, sec);
			}
		}
	}
}

int
main(int ac, char **av)
{
	int div4, delta_sum;
	int i, n, v, hi, low, note, velo;
	int ch, type;
	double sec;
	struct out_rec otr;
	int tempo, tempo_delta_sum;
	double tempo_sec;

	out_init(&otr, ac, av);
	note_buf_init();
	stat_buf_init();
	for(i=0; i<MIDI_CH_N; i++){
		ch_inf[i].vol = 0;
		ch_inf[i].pan = 1<<(14-1);
		ch_inf[i].bend = 0;
		ch_inf[i].rpn = 0;
		ch_inf[i].bend_range = 2;
		ch_inf[i].prog =-1;
	}
	tempo = 500000;
	tempo_delta_sum = 0;
	tempo_sec = 0;

	div4 = header();

	if(!rd_str_chk(4, "MTrk")) ERR("track id");
	v = rd_int(4); /* skip */

	hi = low = 0;
	delta_sum = 0;
	while((v = rd_delta()) != EOF){
		delta_sum += v;
		sec = tempo_sec + (double)(delta_sum - tempo_delta_sum) / div4 * tempo * 0.000001;
		
		if((v = rd()) & 0x80){
			hi = (v >> 4) & 0xf;
			low  = v & 0xf;
		}else bk(v);

		data_out(&otr, sec);

		ch = low;
		switch(hi){
		case 8:
		case 9:
			note = rd();
			velo = rd();
			note_onoff(hi == 9, sec, ch, note, velo);
			break;
		case 0xb: /* control change */
			type = rd();
			v = rd();
			switch(type){
			case 7: /* channel volume msb */
				msb_set(&ch_inf[ch].vol, v);
				break;
			case 39: /* channel volume lsb */
				lsb_set(&ch_inf[ch].vol, v);
				break;
			case 10: /* pan msb */
				msb_set(&ch_inf[ch].pan, v);
				break;
			case 42: /* pan lsb */
				lsb_set(&ch_inf[ch].pan, v);
				break;
			case 100: /* rpn lsb */
				lsb_set(&ch_inf[ch].rpn, v);
				break;
			case 101: /* rpn msb */
				msb_set(&ch_inf[ch].rpn, v);
				break;
			case 6: /* data entry msb */
				switch(ch_inf[ch].rpn){
				case 0: /* pitch bend range */
					ch_inf[ch].bend_range = v;
					break;
				}
				break;
			}
			break;
		case 0xa:
			rd();
			rd();
			break;
		case 0xe: /* pitch wheel change */
			bend_note_update(ch, sec);
			v = rd();
			v |= rd() << 7;
			ch_inf[ch].bend = v - 8192;
			break;
		case 0xc: /* program number */
			v = rd();
			ch_inf[ch].prog = v;
			break;
		case 0xd:
			rd();
			break;
		case 0xf:
			type = rd();
			switch(low){
			case 0:
				while(rd() != 0xf7);
				break;
			case 1:
			case 3:
				rd();
				break;
			case 2:
				rd();
				rd();
				break;
			case 0xf: /* meta */
				n = rd();
				switch(type){
				case 0x51: /* set tempo */
					v = rd_int(n);
					tempo = v;
					tempo_delta_sum = delta_sum;
					tempo_sec = sec;
					break;
				default:
					for(i=0; i<n; i++) rd();
					break;
				}
				break;
			default:
				break;
			}
			break;
		}
	}
	if(otr.fp != stdout) pclose(otr.fp);
	return 0;
}

