#include "note.h"
#include "stat.h"
#include "ch.h"

struct note_rec note_buf[NOTE_BUF_N];

int
note_buf_is_free(int i)
{
	return note_buf[i].ch < 0;
}

void
note_buf_free(int i)
{
	note_buf[i].ch = -1;
}

void
note_buf_init(void)
{
	int i;

	for(i=0; i<NOTE_BUF_N; i++) note_buf_free(i);
}

int
note_buf_search(int ch, int note, int onoff)
{
	int i;

	for(i=0; i<NOTE_BUF_N; i++) {
		if( (ch < 0 && note_buf[i].ch < 0) ||
		    (note_buf[i].ch == ch &&
		     note_buf[i].note == note &&
		     note_buf[i].onoff == onoff) ) return i;
	}
	return -1;
}

double
note_to_freq(int note)
{
	return 440 * pow(2, (note - 69) / 12.0);
}

int
note_out(struct note_rec *nt, double *vl, double *vr)
{
	double v, sum_v, freq, pan, env_v, d_env_v;
	int nch, cnt;
	struct tone_rec *tn;
	struct tone_compo_rec *tone_compo;
	struct stat_rec *stat;
	struct out_rec *ot;

	nch = nt->ch;
	ot = nt->ot;
	cnt = 0;
	sum_v = 0;
	for(stat=nt->stat_lst; stat; stat=stat->next){
		tone_compo = stat->tone_compo;
		if(tone_compo == NULL) continue;
		cnt++;
		tn = tone_compo->tone;
		freq = note_to_freq(tone_compo->note >= 0 ? tone_compo->note : nt->note);

		if(stat->fst_smp_cnt < 0) stat->fst_smp_cnt = ot->smp_cnt;
		if(ch_inf[nch].bend != 0) freq *= pow(2, ch_inf[nch].bend * ch_inf[nch].bend_range * (1.0 / (8192 * 12)));

		env_v = env_out(stat, ot->sec);
		v = (env_v > 0) ? tone_out(stat, freq, env_v) : 0;

		d_env_v = 0;
		if(tn->delay.onoff) v += delay_out(stat, &tn->delay, &d_env_v);
		d_env_v += env_v;
		delay_buf_set(stat, v, d_env_v);
		if(tn->chorus) v += chorus_out(stat);

		sum_v += v * tone_compo->rate;

		if(nt->onoff == 0 && ot->sec - nt->off_sec >= tn->env.release && (!tn->delay.onoff || d_env_v < 0.01)){
			stat->tone_compo = NULL;
		}
	}
	v = sum_v;

	v *= (nt->velo * (1.0 / 127)) * (ch_inf[nch].vol * (1.0 / ((1<<14)-1)));
	if(ot->ch_num == 1){
		*vl += v;
	}else{
		pan = ch_inf[nch].pan * (1.0 / (1<<14));
		*vl += (1 - pan) * v;
		*vr += pan * v;
	}
	return cnt == 0;
}

void
note_onoff(int onoff, double evt_sec, int ch, int note, int velo)
{
	int i, tone_n;
	struct note_rec *nt;
	struct stat_rec *stat;
	struct tone_compo_rec *tone_compo;

	if(onoff){
		if((i = note_buf_search(-1, -1, -1)) < 0){
			MSG("note_buf full");
			return;
		}
		nt = &note_buf[i];
		nt->note = note;
		nt->ch = ch;
		nt->onoff = 1;
		nt->on_sec = evt_sec;
		nt->velo = velo;

		if((tone_compo = tone_compo_get(ch, note, &tone_n)) == NULL){
			note_buf_free(i);
			return;
		}
		if((nt->stat_lst = stat_lst_alloc_init(tone_n, evt_sec, nt, tone_compo, NULL)) == NULL){
			note_buf_free(i);
		}
	}else{
		if((i = note_buf_search(ch, note, 1)) < 0) return;
		nt = &note_buf[i];
		for(stat=nt->stat_lst; stat; stat=stat->next){
			stat->off_v = env_out(stat, evt_sec);
		}
		nt->onoff = 0;
		nt->off_sec = evt_sec;
	}
}

/* EOF */
