#include "out.h"

int
sox_version(void)
{
	FILE *fp;
	int v1, v2, v3;

	if((fp = popen("sox --version | sed -e 's/^.*v//'", "r")) == NULL) return -1;
	fscanf(fp, "%d.%d.%d", &v1, &v2, &v3);
	pclose(fp);
	return v1*10000 + v2*100 + v3;
}

char *
sox_bit_len_fmt(int bit_len)
{
	return sox_version() >= 140400 ? 
		(bit_len == 8 ? "-b 8" : "-b 16") : 
		(bit_len == 8 ? "-b" : "-w");
}

void
out_init(struct out_rec *ot, int ac, char **av)
{
	char cmd[1024], *fn;

	ot->smp_freq = opt_int("-r", ac, av, 44100);
	ot->smp_cnt = 0;
	ot->bit_len = opt_int("-b", ac, av, 16);
	if(ot->bit_len != 8 && ot->bit_len != 16) ERR("bit_len");
	ot->sign = (opt_idx("-u", ac, av) < 0);
	ot->ch_num = opt_int("-c", ac, av, 2);

	ot->fp = stdout;
	cmd[0] = '\0';
	fn = "";
	if(opt_idx("-play", ac, av) >= 0){
		strcat(cmd, "play");
	}else if(opt_idx("-sox", ac, av) >= 0){
		strcat(cmd, "sox");
		fn = opt_str("-sox", ac, av, "-d");
	}
	if(cmd[0]){
		sprintf(cmd + strlen(cmd), " -t raw -r %d %s %s -c %d - %s",
			ot->smp_freq, sox_bit_len_fmt(ot->bit_len),
			ot->sign ? "-s" : "-u",
			ot->ch_num, fn);
		if((ot->fp = popen(cmd, "w")) == NULL) ERR("popen");
	}
	ot->amp = ot->bit_len == 8 ? 127 : 32767;
	ot->base = ot->sign ? 0 : ot->amp + 1;
	ot->iv_min = ot->base - (ot->amp + 1);
	ot->iv_max = ot->base + ot->amp;

	ot->smp_t = 1.0 / ot->smp_freq;
}

void
out_do(struct out_rec *ot, double v)
{
	int iv;

	iv = ot->base + (int)(v * ot->amp);
	if(iv > ot->iv_max) iv = ot->iv_max;
	if(iv < ot->iv_min) iv = ot->iv_min;
	switch(ot->bit_len){
	case 8:
		fputc(iv, ot->fp);
		break;
	case 16:
		fputc(iv & 0xff, ot->fp);
		fputc((iv >> 8) & 0xff, ot->fp);
		break;
	}
}

/* EOF */

