#include "tone.h"
#include "ch.h"
#include "vcf.h"
#include "stat.h"

struct tone_rec tone_inf[] = {
	{	/* strings */
		{ WAVE_SAW, WAVE_SAW, 12, 0.5, OFF },
		{ LPF, 3000, 1.5 }, { OFF, },
		{ 0.15, 0.5, 0.8, 0.5 },
		1.0,
		{ OFF, 12, OFF, OFF }, { WAVE_SIN, 4, 0.3 },
		{ OFF, OFF, OFF, OFF },
		{ OFF, }, ON,
	},{	/* bass */
		{ WAVE_SAW, WAVE_SQUARE, OFF, 0.5, OFF },
		{ LPF, 1000, 1.5 }, { OFF, },
		{ 0.05, 0.2, 0.4, 0.2 },
		1.0,
		{ OFF, 60, OFF, OFF }, { WAVE_SIN, 1.5, 0.2 },
		{ OFF, OFF, OFF, OFF },
		{ OFF, }, ON,
	},{	/* lead */
		{ WAVE_SQUARE, WAVE_SQUARE, 10, 0.5, OFF },
		{ LPF, 2000, 4 }, { OFF, },
		{ 0.01, 0.2, 0.8, 0.3 },
		1.0,
		{ OFF, 1, 5, OFF }, { WAVE_SIN, 4, 0.3 },
		{ OFF, OFF, OFF, OFF },
		{ ON, 0.2, 0.4}, OFF,
	},{	/* SIN */
		{ WAVE_SIN, WAVE_SIN, OFF, 0, OFF },
		{ LPF, 20000, 0.8 }, { OFF, },
		{ 0, 0.3, 0.2, 0.3 },
		1.4,
		{ 25, OFF, OFF, OFF }, { WAVE_SIN, 6, 0.3 },
		{ OFF, OFF, OFF, OFF },
		{ OFF, }, OFF,
	}
}, drum_tone_inf[] = {
	{	/* bass */
		{ WAVE_SIN, WAVE_NOISE, OFF, 0.4, OFF },
		{ LPF, 400, 1 }, { OFF, },
		{ 0.01, 0.18, 0, 0.18 },
		9.0,
		{ 100, OFF, 30, OFF }, { WAVE_NOISE, 0, 0.5 },
		{ OFF, OFF, OFF, OFF },
		{ OFF, }, OFF,
	},{	/* snare */
		{ WAVE_SQUARE, WAVE_NOISE, OFF, 0.8, ON },
		{ LPF, 1500, 1.7 }, { OFF, }, 
		{ 0, 0.4, 0.3, 0.4 },
		5,
		{ OFF, OFF, OFF, OFF }, {},
		{ OFF, OFF, OFF, OFF },
		{ OFF, }, OFF,
	},{	/* tom */
		{ WAVE_SQUARE, WAVE_NOISE, OFF, 0.5, ON },
		{ LPF, 800, 2.5 }, { OFF, },
		{ 0, 0.03, 0.1, 0.4 },
		5,
		{ OFF, OFF, OFF, OFF }, {},
		{ 200, OFF, 1200, OFF },
		{ OFF, }, OFF,
	},{	/* hi-hat close */
		{ WAVE_SQUARE, WAVE_SQUARE, 700, 0.9, ON },
		{ LPF, 16000, 2 }, { OFF, },
		{ 0, 0.15, 0, 0.15 },
		0.6,
		{ OFF, OFF, OFF, OFF }, {},
		{ OFF, OFF, OFF, OFF },
		{ OFF, }, OFF,
	},{	/* hi-hat open */
		{ WAVE_SQUARE, WAVE_SQUARE, 700, 0.9, ON },
		{ LPF, 16000, 2 }, { OFF, },
		{ 0, 0, 1, 0.35 },
		0.6,
		{ OFF, OFF, OFF, OFF }, {},
		{ OFF, OFF, OFF, OFF },
		{ OFF, }, OFF,
	},{	/* cymbal */
		{ WAVE_NOISE, WAVE_SQUARE, OFF, 0.8, ON },
		{ LPF, 4000, 2 }, { OFF, },
		{ 0, 0.01, 0.2, 0.8 },
		9,
		{ OFF, OFF, OFF, OFF }, {},
		{ OFF, OFF, 600, OFF }, 
		{ OFF, }, OFF,
	},{	/* side stick */
		{ WAVE_NOISE, WAVE_SQUARE, OFF, 0.2, OFF },
		{ LPF, 1700, 3 }, { OFF, },
		{ 0, 0.012, 0.1, 0.7 },
		10,
		{ OFF, OFF, OFF, OFF }, {},
		{ OFF, OFF, OFF, OFF },
		{ OFF, }, OFF,
	}
};

#define PROG_DRUM	0

struct{
	int prog, note; /* note for ch 9 */
	struct tone_compo_rec *tone_compo;
} tones_lst[] = {
	{
		PROG_DRUM, 36, /* bass drum1 */
		(struct tone_compo_rec []){
			{ &drum_tone_inf[0], 28, 1.0 }, { NULL, }
		}
	},{
		PROG_DRUM, 37, /* side stick */
		(struct tone_compo_rec []){
			{ &drum_tone_inf[6], 110, 1.0 }, { NULL, }
		}
	},{
		PROG_DRUM, 40, /* electric snare */
		(struct tone_compo_rec []){
			{ &drum_tone_inf[1], 69, 1.0 }, { NULL, }
		}
	},{
		PROG_DRUM, 41, /* low floor tom */
		(struct tone_compo_rec []){
			{ &drum_tone_inf[2], 50, 1.0 }, { NULL, }
		}
	},{
		PROG_DRUM, 42, /* closed hi-hat */
		(struct tone_compo_rec []){
			{ &drum_tone_inf[3], 115 + 8, 1.0 }, { NULL, }
		}
	},{
		PROG_DRUM, 46, /* open hi-hat */
		(struct tone_compo_rec []){
			{ &drum_tone_inf[4], 115 + 8, 1.0 }, { NULL, }
		}
	},{
		PROG_DRUM, 49, /* crash cymbal 1 */
		(struct tone_compo_rec []){
			{ &drum_tone_inf[5], 75, 1.0 }, { NULL, }
		}
	},{
		PROG_DRUM, 57, /* crash cymbal 2 */
		(struct tone_compo_rec []){
			{ &drum_tone_inf[5], 85, 1.0 }, { NULL, }
		}
	},{
		48, -1, /* timpani */
		(struct tone_compo_rec []){
			{ &tone_inf[0], -1, 1.0 }, { NULL, }
		}
	},{
		50, -1, /* strings ensamble 2 */
		(struct tone_compo_rec []){
			{ &tone_inf[0], -1, 1.0 }, { NULL, }
		}
	},{
		35, -1, /* electric bass (pick) */
		(struct tone_compo_rec []){
			{ &tone_inf[1], -1, 1.0 }, { NULL, }
		}
	},{
		79, -1, /* whistle */
		(struct tone_compo_rec []){
			{ &tone_inf[2], -1, 1.0 }, { NULL, }
		}
	},{
		81, -1, /* lead 1 (square) */
		(struct tone_compo_rec []){
			{ &tone_inf[2], -1, 1.0 }, { NULL, }
		}
	},{
		87, -1, /* lead 7 (fifths) */
		(struct tone_compo_rec []){
			{ &tone_inf[2], -1, 1.0 }, { NULL, }
		}
	},{
		24, -1, /* tango accordion */
		(struct tone_compo_rec []){
			{ &tone_inf[3], -1, 1.0 }, { NULL, }
		}
	},{
		67, -1, /* tenor sax */
		(struct tone_compo_rec []){
			{ &tone_inf[3], -1, 1.0 }, { NULL, }
		}
	},{
		-1, /* tail */
	}
};

#define CH_PROG(ch)	( (ch) == 9 ? 0 : ch_inf[ch].prog )

struct tone_compo_rec *
tone_compo_get(int ch, int note, int *ret_n)
{
	struct tone_compo_rec *tone_compo;
	int prog, i;

	prog = CH_PROG(ch);
	for(i=0; tones_lst[i].prog >= 0; i++){
		if(tones_lst[i].prog == prog &&
		   (tones_lst[i].note == note || tones_lst[i].note < 0)) break;
	}
	if(tones_lst[i].prog < 0) return NULL; /* not found */
	tone_compo = tones_lst[i].tone_compo;
	if(ret_n){
		for(i=0; tone_compo[i].tone; i++);
		*ret_n = i;
	}
	return tone_compo;
}

double
tone_out(struct stat_rec *stat, double freq, double env_v)
{
	struct tone_rec *tn;
	int i;
	double v, v1, v2, lfo_v, modu_v[4];
	struct vco_rec *vco;
	struct out_rec *ot;

	tn = stat->tone_compo->tone;
	ot = stat->nt->ot;

	lfo_v = lfo_out(&tn->lfo, &tn->lfo_modu, ot->sec, stat->nt->on_sec);

	for(i=0; i<4; i++) modu_v[i] = 0;
	modu_out(lfo_v, &tn->lfo_modu, modu_v);
	modu_out(env_v, &tn->env_modu, modu_v);

	vco = &tn->vco;
	v1 = vco_out(vco->wave1, &stat->vco[0], freq, ot->sec, modu_v[0]);

	modu_v[1] += vco->tune * (1.0 / 1200);
	v2 = vco_out(vco->wave2, &stat->vco[1], freq, ot->sec, modu_v[1]);
	if(vco->ring){
		v = v1 * v2;
		v1 = (v1 + v2) * 0.5;
		v2 = v;
	}
	v = MIX(v1, v2, vco->mix);

	if(tn->fl1.type != OFF) v = vcf_out(&tn->fl1, &stat->filter[0], ot->smp_t, modu_v[2], v);
	if(tn->fl2.type != OFF) v = vcf_out(&tn->fl2, &stat->filter[1], ot->smp_t, modu_v[3], v);

	v *= env_v * tn->level;
	return v;
}

/* EOF */
